/**
 * Created by sharyu on 16/11/21.
 */

var __failCount = 0;

var AssetsManagerLoaderScene = cc.Scene.extend({
    _am:null,
    _progress:null,
    _notice:null,
    _percent:0,
    _percentByFile:0,

    ctor : function(){
        this._super();
    },

    onEnter : function () {
        cc.Scene.prototype.onEnter.call(this);

        if (!cc.sys.isNative || DEBUG_WITHOUT_ASSETMANAGER) {
            this.loadGame();
            return;
        }

        var layer = new cc.Layer();
        this.addChild(layer);

        this._notice = new cc.LabelTTF("", "Arial", 24);
        this._notice.x = 30;
        this._notice.y = cc.winSize.height - 200;
        this._notice.setHorizontalAlignment(cc.TEXT_ALIGNMENT_LEFT);
        this._notice.setAnchorPoint(cc.p(0, 0.5));
        layer.addChild(this._notice);

        this._progress = new cc.LabelTTF("0%", "Arial", 24);
        this._progress.x = cc.winSize.width / 2;
        this._progress.y = cc.winSize.height / 2 - 50;
        layer.addChild(this._progress);

        var storagePath = (jsb.fileUtils ? jsb.fileUtils.getWritablePath() : "./");

        this._am = new jsb.AssetsManager("res/project.manifest", storagePath);
        this._am.retain();

        if (!this._am.getLocalManifest().isLoaded())
        {
            this.showNotice("Fail to update assets, step skipped.");
            that.showRestartDialog();
        }
        else
        {
            var that = this;
            var listener = new jsb.EventListenerAssetsManager(this._am, function(event) {
                switch (event.getEventCode()){
                    case jsb.EventAssetsManager.ERROR_NO_LOCAL_MANIFEST:
                        that.showNotice("No local manifest file found, skip assets update.");
                        that.showRestartDialog();
                        break;
                    case jsb.EventAssetsManager.UPDATE_PROGRESSION:
                        that._percent = event.getPercent();
                        that._percentByFile = event.getPercentByFile();

                        var msg = event.getMessage();
                        if (msg) {
                            that.showNotice(msg);
                        }
                        break;
                    case jsb.EventAssetsManager.ERROR_DOWNLOAD_MANIFEST:
                    case jsb.EventAssetsManager.ERROR_PARSE_MANIFEST:
                        that.showNotice("Fail to download manifest file, update skipped.");
                        that.showRestartDialog();
                        break;
                    case jsb.EventAssetsManager.ALREADY_UP_TO_DATE:
                    case jsb.EventAssetsManager.UPDATE_FINISHED:
                        that.showNotice("Update finished.");
                        that.loadGame();
                        break;
                    case jsb.EventAssetsManager.UPDATE_FAILED:
                        that.showNotice("Update failed. " + event.getMessage());

                        __failCount ++;
                        if (__failCount < 5)
                        {
                            that._am.downloadFailedAssets();
                        }
                        else
                        {
                            that.showNotice("Reach maximum fail count, exit update process");
                            __failCount = 0;
                            that.showRestartDialog();
                        }
                        break;
                    case jsb.EventAssetsManager.ERROR_UPDATING:
                        that.showNotice("Asset update error: " + event.getAssetId() + ", " + event.getMessage());
                        //that.showRestartDialog();
                        break;
                    case jsb.EventAssetsManager.ERROR_DECOMPRESS:
                        that.showNotice(event.getMessage());
                        that.showRestartDialog();
                        break;
                    case jsb.EventAssetsManager.ASSET_UPDATED:
                        that._percent = event.getPercent();
                        that._percentByFile = event.getPercentByFile();

                        that.showNotice("ASSET_UPDATED:"+event.getAssetId() + ", " + that._percentByFile);
                        break;
                    default:
                        that.showNotice("code:"+event.getEventCode()+", msg:" + event.getMessage());
                        break;
                }
            });

            cc.eventManager.addListener(listener, 1);
            this._am.update();
            cc.log("start to update assets...");
        }

        this.schedule(this.updateProgress, 0.5);
    },

    onExit:function(){
        cc.Scene.prototype.onExit.call(this);

        if (cc.sys.isNative) {
            if (this._am != null) {
                this._am.release();
            }
        }
    },

    restartGame : function () {
        this.runAction(cc.sequence(cc.delayTime(1.0), cc.callFunc(new function() {
            //cc.game.restart();
            var asm = new AssetsManagerLoaderScene();
            cc.director.runScene(asm);
        })));
    },

    loadGame:function(){
        cc.loader.loadJs(["src/jsFiles.js"], function(err){
            cc.loader.loadJs(jsFiles, function(err){

                ////测试ui
                if(cc.sys.platform != cc.sys.DESKTOP_BROWSER && cc.sys.platform != cc.sys.MOBILE_BROWSER){
                    sa.LoaderScene.preloadEx=sa.LoaderScene.preload;
                }
                sa.LoaderScene.preload(g_resources, function () {
                    sa.LoaderScene.preload(g_fonts, function () {
                        sa.LoaderScene.preload(g_resMenu, function () {
                            sa.LoaderScene.preloadEx(g_resMyPage, function () {
                                if (DEBUG_WITHOUT_DMM) {
                                    SaveData.shared().loadKeyChain();
                                }
                                if(DEBUG_SCRIPT)
                                {
                                    var scene = new ScriptTestInputScene();
                                    cc.director.runScene(scene);
                                }
                                else{
                                    SaveData.shared().loadVersionInfo();
                                    cc.director.runScene(new TitleConnectScene());
                                }
                            }, this, "mypage");
                        }, this, "menu");
                    },this);
                }, this);
            });
        });
    },

    updateProgress:function(dt){
        this._progress.string = "" + this._percent;
    },

    showNotice : function (str) {
        this._notice.string = str;
        cc.log(str);
    },

    showRestartDialog : function () {
        var colorlayer = new cc.LayerColor(cc.color(255, 255, 255, 250));
        colorlayer.setContentSize(cc.size(300, 300));
        colorlayer.setPosition(cc.p(cc.winSize.width/2-150, cc.winSize.height/2-150));
        this.addChild(colorlayer, 999);
        colorlayer.setAnchorPoint(cc.p(0.5, 0.5));

        var item = new cc.MenuItemFont("Restart", this.restartGame, this);
        item.setColor(cc.color(255, 0, 0));
        var menu = new cc.Menu(item);
        menu.setPosition(cc.p(150, 100));
        colorlayer.addChild(menu);
    },
});